from pathlib import Path


class Settings(object):
    LOCALES = {
        "en": "English",
        "ru": "Русский",
    }
    SECRET_KEY = "kjmhg345bjknhg7jhKGM98KNJKv1nMKJH3YNG2bjvn326jmh"

    SITE_TITLE = "FogStone"
    CONTENT_DIR = Path(__file__).parent / Path("data")
    TEMPLATE_FOLDER = Path("templates")
    STATIC_FOLDER = Path("static")

    TRANSLIT_PERMALINKS = False
    TRANSLIT_CODE = "ru"
    TRANSLIT_REVERSE = True
    SEARCH_ENABLED = False



import re
from os import environ
from typing import Callable
from pathlib import Path

from flask import Flask, Config, request, g
import flask_babel

from markdown import Markdown
from pyembed.markdown import PyEmbedMarkdown
from markdown.extensions.toc import TocExtension
import unicodedata
from transliterate.decorators import transliterate_function


from fogstone import views
from fogstone.settings import Settings

__all__ = ["create_app"]


def configure_views(app: Flask):
    app.add_url_rule("/", "index", views.index)
    app.add_url_rule("/<path:raw_path>", "content", views.content)
    app.add_url_rule("/search", "search", views.search)


def configure_i18n(app: Flask):
    babel = flask_babel.Babel(app)

    @babel.localeselector
    def get_locale():
        if not request:
            if hasattr(g, 'locale'):
                return g.locale
            raise RuntimeError('Babel is used outside of request context, please set g.locale')
        locales = app.config['LOCALES'].keys()
        locale = request.cookies.get('locale')
        if locale in locales:
            return locale
        return request.accept_languages.best_match(locales)

    @app.before_request
    def before_request():
        g.locale = flask_babel.get_locale()


def get_slugify(config: Config) -> Callable[[str, str], str]:
    def func(value: str, separator: str):
        value = unicodedata.normalize("NFKD", value)
        value = re.sub(r"[^\w\s-]", "", value).strip().lower()
        return re.sub(r"[%s\s]+" % separator, separator, value)

    if config["TRANSLIT_PERMALINKS"]:
        return transliterate_function(
            language_code=config["TRANSLIT_CODE"],
            reversed=config["TRANSLIT_REVERSE"]
        )(func)
    else:
        return func


def configure_md(app: Flask):
    app.MD = Markdown(extensions=[
        "markdown.extensions.meta",
        "markdown.extensions.extra",
        "markdown.extensions.sane_lists",
        "markdown.extensions.toc",
        PyEmbedMarkdown(),
        TocExtension(
            baselevel=2,
            permalink=True,
            slugify=get_slugify(app.config),
            separator="_"
        )
    ])


def make_config() -> Settings:
    cfg = Settings()
    if "FOGSTONE_SECRET_KEY" in environ:
        cfg.SECRET_KEY = environ.get("FOGSTONE_SECRET_KEY")
    if "FOGSTONE_SITE_TITLE" in environ:
        cfg.SITE_TITLE = environ.get("FOGSTONE_SITE_TITLE")
    if "FOGSTONE_CONTENT_DIR" in environ:
        cfg.CONTENT_DIR = Path(environ.get("FOGSTONE_CONTENT_DIR"))
    if "FOGSTONE_TEMPLATE_FOLDER" in environ:
        cfg.TEMPLATE_FOLDER = Path(environ.get("FOGSTONE_TEMPLATE_FOLDER"))
    if "FOGSTONE_STATIC_FOLDER" in environ:
        cfg.STATIC_FOLDER = Path(environ.get("FOGSTONE_STATIC_FOLDER"))

    return cfg


def create_app() -> Flask:
    config_object = make_config()
    app = Flask(
        __name__,
        template_folder=config_object.TEMPLATE_FOLDER.as_posix(),
        static_folder=config_object.STATIC_FOLDER.as_posix(),
    )
    app.config.from_object(config_object)
    configure_views(app)
    configure_i18n(app)
    configure_md(app)

    return app



from flask import render_template, abort, request, current_app
from flask_babel import gettext

from fogstone.logic import construct_path, read_content, read_hierarchy, locate_page


def content(raw_path):
    path = construct_path(raw_path)
    if path is None:
        abort(404)

    page = read_content(path)

    hierarchy = read_hierarchy(current_app.config["CONTENT_DIR"], recursive=True)
    sidebar = locate_page(hierarchy, page)

    return render_template(
        "page.html",
        site_title=current_app.config["SITE_TITLE"],
        page_title=page.meta.title,
        page=page,
        sidebar=sidebar,
        search_enabled=current_app.config["SEARCH_ENABLED"],
    )


def index():
    path = construct_path("/index")
    if path is None:
        abort(404)

    page = read_content(path)

    return render_template(
        "page.html",
        site_title=current_app.config["SITE_TITLE"],
        page_title=page.meta.title,
        page=page,
        search_enabled=current_app.config["SEARCH_ENABLED"],
    )


def search():
    if not current_app.config["SEARCH_ENABLED"]:
        abort(404)

    search_request = request.args.get("q")

    sidebar = read_hierarchy(current_app.config["CONTENT_DIR"], recursive=True)

    return render_template(
        "search.html",
        site_title=current_app.config["SITE_TITLE"],
        page_title=gettext("Search"),
        sidebar=sidebar,
        results=[],
        search=search_request,
        search_enabled=current_app.config["SEARCH_ENABLED"],
    )



import re
from datetime import datetime
from pathlib import Path
from typing import Optional

from flask import current_app

from fogstone.models import Meta, Page, Hierarchy

PATH_REGEX = re.compile("^[\w_-]+$")
SPEC_SYMBOL_REGEX = re.compile("[\W_-]")
LOCAL_ROOT = Path(".")


def construct_path(raw_path: str) -> Optional[Path]:
    raw_chunks = [c for c in raw_path.split("/") if c]
    if not all(PATH_REGEX.match(chunk) for chunk in raw_chunks):
        return None
    naive_path = (current_app.config["CONTENT_DIR"] / Path(*raw_chunks)).resolve()

    if naive_path.exists():
        return naive_path

    naive_path = naive_path.with_suffix(".md")

    if naive_path.exists():
        return naive_path


def make_link(path: Path) -> str:
    GLOBAL_ROOT = current_app.config["CONTENT_DIR"]
    p = path.parent if path.stem == "index" else path

    if p == LOCAL_ROOT or p == GLOBAL_ROOT:
        return "/"

    return ("/" / p.relative_to(GLOBAL_ROOT)).with_suffix('').as_posix()


def title_from_path(path: Path) -> str:
    return SPEC_SYMBOL_REGEX.sub(" ", path.stem).capitalize()


def mtime_from_path(path: Path) -> str:
    return datetime.fromtimestamp(path.stat().st_mtime).strftime("%Y-%m-%d")


def read_hierarchy(path: Path, recursive=False) -> Hierarchy:
    title = title_from_path(path)
    link = make_link(path)
    children = [
        read_hierarchy(p, recursive=recursive)
        for p in path.glob("*")
        if (True if recursive else p.is_file())
        and p.stem.lower() != "index"
    ]
    return Hierarchy(title=title, link=link, children=children)


def locate_page(hierarchy: Hierarchy, page: Page) -> Hierarchy:
    return Hierarchy(
        title=hierarchy.title,
        link=hierarchy.link,
        current=page.link == hierarchy.link,
        children=[locate_page(h, page) for h in hierarchy.children]
    )


def read_file(path: Path) -> Page:
    raw_content = path.read_text("utf-8")
    html = current_app.MD.convert(raw_content)
    meta = Meta(
        title=current_app.MD.Meta.get("title", [title_from_path(path)])[0],
        description=current_app.MD.Meta.get("description", [None])[0],
        authors=current_app.MD.Meta.get("authors", []),
        date=current_app.MD.Meta.get("date", [mtime_from_path(path)])[0],

        hide_hierarchy="hide_hierarchy" in current_app.MD.Meta,
        hide_title="hide_title" in current_app.MD.Meta,
    )
    return Page(content=html, meta=meta, link=make_link(path))


def read_catalog(path: Path) -> Page:
    index_path = path / "index.md"

    if index_path.exists():
        return read_file(index_path)
    else:
        return Page(
            meta=Meta(title=title_from_path(path), date=mtime_from_path(path)),
            link=make_link(path),
            hierarchy=read_hierarchy(path)
        )


def read_content(path: Path) -> Page:
    return read_catalog(path) if path.is_dir() else read_file(path)



from pathlib import Path
from typing import Optional, List
from typing import ForwardRef

from pydantic import BaseModel


class Meta(BaseModel):
    title: str
    description: Optional[str] = None
    authors: List[str] = []
    date: Optional[str] = None
    hide_hierarchy: bool = False
    hide_title: bool = False


Hierarchy = ForwardRef('Hierarchy')


class Hierarchy(BaseModel):
    title: str
    link: str
    current: bool = False
    children: List[Hierarchy] = []


class Page(BaseModel):
    meta: Meta
    content: str = ""
    excerpt: Optional[str] = None
    hierarchy: Optional[Hierarchy] = None
    link: str = "#"


Hierarchy.update_forward_refs()


class Config(BaseModel):
    locale: str
    site_title: str
    copyright: Optional[str] = None
    content_dir: Path



import os

from fogstone.app import create_app

host = os.getenv('HOST', "127.0.0.1")
port = os.getenv('PORT', "5000")
debug = 'DEBUG' in os.environ

app = create_app()


def run():
    app.run(debug=debug, host=host, port=int(port))


if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
from setuptools import setup, find_packages


if sys.version_info < (3, 6):
    print("FogStone requires Python 3.6 or later.")
    sys.exit(1)

import fogstone


setup(
    name="fogstone",
    version=fogstone.__version__,
    description="Simple helpdesk system",
    long_description=open("README.rst").read(),
    author="Andriy Kushnir (Orhideous)",
    author_email="me@orhideous.name",
    license="MIT",
    url="https://github.com/MC4EP/fogstone",
    platforms="any",
    packages=find_packages(),
    install_requires=open("requirements.txt").read().splitlines(),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "fogstone=fogstone.__main__:run",
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: Flask",
        "License :: OSI Approved",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
    ],
)
